import sys
from passgenpyNYU import core
import argparse

def positive_int(val):
    try:
        value = int(val)
    except ValueError: 
        raise argparse.ArgumentTypeError("Invalid input. Must be a whole number greater than 0")
    
    if value <= 0: 
        raise argparse.ArgumentTypeError("Invalid Value. Must be greater than 0")
    
    return value

def parser():
    parser = argparse.ArgumentParser(
        description="Generate a funny or serious password"
    )

    parser.add_argument(
        "-g",
        "--generate",
        dest="generate",
        type= positive_int,
        help= "Generate a random password. Takes the following argument: length, capitalize= False, numbers= False, symbols= False",
    )

    parser.add_argument(
        "-c", 
        "--caesar_cipher",
        nargs=2,
        help= "Decode a string based on caesar cipher. Takes two arguments: string and int (between 1 and 25) for shift",
    )

    parser.add_argument(
        "-s", 
        "--scramble",
        dest= "scramble",
        help= "Scramble a string as a password. Takes one argument: string",
    )

    parser.add_argument(
        "-f",
        "--getFunnyPassword",
        dest= "funnyPassword",
        action= "store_true",
        help= "Get a funny password. No arguments"
    )

    parser.add_argument(
        "-b",
        "--binarypassword",
        dest= "binarypassword",
        type= positive_int,
        help= "Generate a binary password. Takes in length of string as argument."
    )

    parser.add_argument(
        "-gh",
        "--generatehash",
        dest= "generatehash",
        help= "Converts password into Hash. Takes in string as arugment"
    )

    return parser


def main(): 
    parse = parser()

    try:
        args = parse.parse_args()
    except argparse.ArgumentError as exc: 
        print("Error parsing arguments", str(exc))
        exit(-1)

    if args.generate is not None:
        print(core.generate(args.generate))

    if args.caesar_cipher:
        try:
            print(core.caesar_cipher(args.caesar_cipher[0], int(args.caesar_cipher[1])))
        except ValueError:
            print("Error Second Argument is not a valid integer between 1 and 25") 
    
    if args.generatehash:
        print(core.generateHash(args.generatehash))

    if args.binarypassword is not None:
        print(core.binary_password(int(args.binarypassword)))

    if args.scramble:
        print(core.scramble(args.scramble))

    if args.funnyPassword:
        print(core.getFunnyPassword())


if __name__ == "__main__":
    main()


import random
import math
import hashlib

def generate(length):
    '''
    This function generates a password of a given length
    '''
    if type(length) != int:
        raise ValueError("Length Must be an integer!!")
    list_lowercase = 'abcdefghijklmnopqrstuvwxyz'
    all_chars = list_lowercase
    password = [random.choice(list_lowercase)]
    while len(password) < length:
        password.append(random.choice(all_chars))
    return ''.join(password)


#does a cesar cipher on a string
def caesar_cipher(word, shift):
    '''
    This functions implements a caesar cipher on an inputted password, 
    where a shift is inputted to shift the letters by a certain amount.
    '''
    if type(word) != str:
        raise ValueError("Input must be a string!")
    if type(shift) != int:
        raise ValueError("Shift must be an int")
    try:
        tempShift = int(shift)
    except ValueError:
        raise ValueError("Shift must be an int")
    if float(shift) % 1 != 0.0:
        raise ValueError("Shift must be an int")
    if not (1 <= tempShift <= 25):
        raise ValueError("Shift value must be between 1 and 25.")
    def shift_letter(char, shift):
        if char.isalpha():
            base = ord('A') if char.isupper() else ord('a')
            return chr((ord(char) - base + shift) % 26 + base)
        return char

    return ''.join(shift_letter(char, tempShift) for char in word)


def scramble(string):
    '''
    This function scrambles an inputted password and returns the scrambled version.
    '''
    if type(string) != str:
        raise ValueError("Input must be a string!")
    toList = [chr for chr in string]
    retString = ''
    while len(toList) > 0:
        rand = math.floor(random.random() * len(toList))
        letter = toList.pop(rand)
        retString = retString + letter
    return retString

def binary_password(l):
    '''
    This function generates a binary password of a given length.
    '''
    if type(l) != int:
        raise ValueError("Length input must be an integer!")
    retString = ''
    for _ in range(l):
        retString+= str(random.randint(0,1))
    return retString

def generateHash(password):
    '''
    This function generates a hash of an inputted password.
    '''
    if type(password) != str:
        raise ValueError("Password must be a string")
    return hashlib.sha256(password.encode()).hexdigest()

def getFunnyPassword():
    '''
    This function generates a funny password. 
    '''
    funny_list_1 = list('123456789')
    funny_list_2= ["sleepy", "hungry", "tired", "angry", "happy", "sad", "excited", "confused", "bored", "nervous", "excited", "happy", "sad", "angry", "tired", "sleepy", "hungry", "bored", "confused", "nervous"]
    funny_list_3= ["cats", "dogs", "birds", "fish", "snakes", "cows", "horses", "sheep", "goats", "pigs", "chickens", "ducks", "turkeys", "geese", "peacocks", "parrots", "penguins", "ostriches", "eagles", "owls", "sparrows", "roosters", "hens", "peahens", "roosters"]
    funny_list_4= ["orange", "red", "blue", "green", "yellow", "purple", "pink", "brown", "black", "white", "gray", "gold", "silver", "bronze", "copper", "iron", "steel", "gold", "silver", "bronze", "copper", "iron", "steel"]
    return random.choice(funny_list_1) + random.choice(funny_list_2) + random.choice(funny_list_4) + random.choice(funny_list_3)





import pytest
import hashlib
import string
import passgenpyNYU.core as core

def test_generate():
    password = core.generate(10)
    assert isinstance(password, str)
    assert len(password) == 10
    assert any(char.isalpha() for char in password)

def test_generate_1():
    password = core.generate(20)
    assert isinstance(password, str)
    assert len(password) == 20
    assert any(char.isalpha() for char in password)

def test_generate_2():
    password = core.generate(45)
    assert isinstance(password, str)
    assert len(password) == 45
    assert any(char.isalpha() for char in password)

def test_basic_shift_cipher():
    assert core.caesar_cipher("hello", 3) == "khoor"
    assert core.caesar_cipher("abc", 2) == "cde"

def test_case_preservation_cipher():
    assert core.caesar_cipher("HeLLo", 3) == "KhOOr"

def test_non_alpha_chars_cipher():
    assert core.caesar_cipher("hello, world!", 5) == "mjqqt, btwqi!"
    assert core.caesar_cipher("1234", 4) == "1234"

def test_shift_boundaries_cipher():
    assert core.caesar_cipher("z", 1) == "a"
    assert core.caesar_cipher("a", 25) == "z"

def test_invalid_shifts_cipher():
    with pytest.raises(ValueError, match="Shift value must be between 1 and 25."):
        core.caesar_cipher("hello", 0)
    with pytest.raises(ValueError, match="Shift value must be between 1 and 25."):
        core.caesar_cipher("hello", 26)

def test_non_integer_shift_cipher():
    with pytest.raises(ValueError, match="Shift must be an int"):
        core.caesar_cipher("hello", "three")
    with pytest.raises(ValueError, match="Shift must be an int"):
        core.caesar_cipher("hello", 3.5)

def test_empty_string_cipher():
    assert core.caesar_cipher("", 5) == ""

def test_scramble():
    test_cases = ["hello", "shaurya", "asjdkd7832", "nsdkajkkadj!askd&", "kjaibbd88*jsd8"]
    for test in test_cases:
        scrambled_string = core.scramble(test)
        assert len(test) == len(scrambled_string)
        char_list_test = [char for char in test]
        char_list_scrambled = [char for char in scrambled_string]
        for char in char_list_test:
            assert char in char_list_scrambled

def test_empty_string_scramble():
    assert core.scramble("") == ""

def test_single_character_scramble():
    assert core.scramble("a") == "a"

def test_spaces_scramble():
    test_str = "hello world"
    scrambled = core.scramble(test_str)
    assert len(scrambled) == len(test_str)
    assert sorted(scrambled) == sorted(test_str)

def test_binary_password():
    bin_password = core.binary_password(10)
    assert isinstance(bin_password, str)
    assert len(bin_password) == 10
    bitlist = list('01')
    for char in bin_password:
        assert char in bitlist

def test_binary_password_randomness():
    bin_password = core.binary_password(10)
    bin_password2 = core.binary_password(10)
    assert bin_password != bin_password2

def test_binary_password_2():
    bin_password = core.binary_password(4)
    assert isinstance(bin_password, str)
    assert len(bin_password) == 4
    bitlist = list('01')
    for char in bin_password:
        assert char in bitlist

def test_generateHash():
    pass_hash = core.generateHash("password")
    assert isinstance(pass_hash, str)
    assert len(pass_hash) == 64

def test_generateHashRandomness():
    pass_hash = core.generateHash("asjdlasjdahhofhhwba7")
    pass_hash2 = core.generateHash("password")
    assert pass_hash != pass_hash2

def test_generate_hash_correctness():
    password = "NYC9sleepyorangecows"
    hash = core.generateHash(password)
    hash_t = core.generateHash(password)
    assert hash == hash_t

def test_generateHas_input():
    with pytest.raises(ValueError, match="Password must be a string"):
        core.generateHash(123)

def test_getFunnyPassword_type():
    funny_password = core.getFunnyPassword()
    assert isinstance(funny_password, str)
    assert len(funny_password) > 0

def test_getFunnyPassowrd_differences():
    results = [core.getFunnyPassword() for _ in range(5)]
    assert len(results) > 1

def test_getFunnycontents():
    funny_password = core.getFunnyPassword()
    funny_list_1 = list('123456789')
    funny_list_2= ["sleepy", "hungry", "tired", "angry", "happy", "sad", "excited", "confused", "bored", "nervous", "excited", "happy", "sad", "angry", "tired", "sleepy", "hungry", "bored", "confused", "nervous"]
    funny_list_3= ["cats", "dogs", "birds", "fish", "snakes", "cows", "horses", "sheep", "goats", "pigs", "chickens", "ducks", "turkeys", "geese", "peacocks", "parrots", "penguins", "ostriches", "eagles", "owls", "sparrows", "roosters", "hens", "peahens", "roosters"]
    funny_list_4= ["orange", "red", "blue", "green", "yellow", "purple", "pink", "brown", "black", "white", "gray", "gold", "silver", "bronze", "copper", "iron", "steel", "gold", "silver", "bronze", "copper", "iron", "steel"]
    assert any(item in funny_password for item in funny_list_1)
    assert any(item in funny_password for item in funny_list_2)
    assert any(item in funny_password for item in funny_list_3)
    assert any(item in funny_password for item in funny_list_4)

def test_invalid_length_generate():
    with pytest.raises(ValueError, match="Length Must be an integer!!"):
        core.generate("hello")

def test_invalid_scramble():
    with pytest.raises(ValueError, match="Input must be a string!"):
        core.scramble(10)

def test_invalid_binary():
    with pytest.raises(ValueError, match= "Length input must be an integer!"):
        core.binary_password("test")